@@ -19,19 +19,18 @@ def check_link_targets(root: Path) -> int:
 
     total = 0
     for doc, soup in site_docs.items():
-        doc = str(doc)
         for link in soup.find_all("a", attrs={"class": "sphinx-codeautolink-a"}):
             base, id_ = link["href"].split("#")
             if any(base.startswith(s) for s in ("http://", "https://")):
                 if base not in external_site_ids:
-                    soup = BeautifulSoup(sess.get(base).text, "html.parser")
-                    external_site_ids[base] = gather_ids(soup)
+                    sub_soup = BeautifulSoup(sess.get(base).text, "html.parser")
+                    external_site_ids[base] = gather_ids(sub_soup)
                 ids = external_site_ids[base]
             else:
                 ids = site_ids[Path(base)]
             assert id_ in ids, (
                 f"ID {id_} not found in {base}"
-                f" while validating link for `{link.string}` in {doc}!"
+                f" while validating link for `{link.string}` in {doc!s}!"
             )
             total += 1
     return total
@@ -39,4 +38,4 @@ def check_link_targets(root: Path) -> int:
 
 def gather_ids(soup: BeautifulSoup) -> set:
     """Gather all HTML IDs from a given page."""
-    return set(tag["id"] for tag in soup.find_all(id=True))
+    return {tag["id"] for tag in soup.find_all(id=True)}
