@@ -1,8 +1,6 @@
 """Docstring."""
 
-from typing import Optional, Union
-
-from .sub import SubBar, subfoo  # NOQA
+from .sub import SubBar, subfoo  # noqa: F401
 
 
 class Baz:
@@ -28,22 +26,22 @@ class Foo:
 
 
 def bar() -> Foo:
-    """bar test function."""
+    """Bar test function."""
 
 
-def optional() -> Optional[Foo]:
+def optional() -> Foo | None:
     """Return optional type."""
 
 
-def optional_manual() -> Union[None, Foo]:
+def optional_manual() -> None | Foo:
     """Return manually constructed optional type."""
 
 
-def optional_counter() -> Union[Foo, Baz]:
+def optional_counter() -> Foo | Baz:
     """Failing case for incorrect optional type handling."""
 
 
-def compile():
+def compile():  # noqa: A001
     """Shadows built in compile function."""
 
 
